@@ -4,7 +4,7 @@
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
  * Copyright (C) 2004-2005 Roland Clobus <rclobus@bigfoot.com>
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -24,9 +24,6 @@
 #include <math.h>
 #include <ctype.h>
 #include <assert.h>
-#ifdef HAVE_HELP
-#include <libgnome/libgnome.h>
-#endif
 
 #include "aboutbox.h"
 #include "frontend.h"
@@ -1255,7 +1252,8 @@ static void game_settings_cb(void)
 static void help_manual_cb(void)
 {
 	GError *error = NULL;
-	gnome_help_display("pioneers", NULL, &error);
+	gtk_show_uri_on_window(GTK_WINDOW(app_window), "help:pioneers",
+			       GDK_CURRENT_TIME, &error);
 	if (error) {
 		log_message(MSG_ERROR, "%s: %s\n", _("Show the manual"),
 			    error->message);
