@@ -4,7 +4,7 @@
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
  * Copyright (C) 2004,2006 Roland Clobus <rclobus@bigfoot.com>
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -23,9 +23,6 @@
 #include "config.h"
 #include <stdlib.h>
 #include "frontend.h"
-#ifdef HAVE_LIBGNOME
-#include <libgnome/libgnome.h>
-#endif
 #include "common_gtk.h"
 #include "config-gnome.h"
 #include "theme.h"
@@ -202,12 +199,6 @@ void frontend_init_gtk_et_al(int argc, char **argv)
 		g_print("\n");
 		exit(0);
 	}
-#if defined(HAVE_HELP) && defined(HAVE_LIBGNOME)
-	gnome_program_init(PACKAGE, FULL_VERSION,
-			   LIBGNOME_MODULE, argc, argv,
-			   GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
-#endif				/* HAVE_HELP && HAVE_LIBGNOME */
-
 #if ENABLE_NLS
 	/* Gtk+ handles the locale, we must bind the translations */
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -263,7 +254,7 @@ void frontend_init(void)
 	notification_init();
 	gui_build_interface();
 
-	/* in theory, all windows are created now... 
+	/* in theory, all windows are created now...
 	 *   set logging to message window */
 	log_set_func_message_window();
 
