@@ -32,6 +32,14 @@
 #include "cards.h"
 #include "log.h"
 
+/* The macro is a modification of the macro in the glib source
+ * The modification allows for const pointers
+ */
+#ifndef G_STRUCT_MEMBER_CONST
+#define G_STRUCT_MEMBER_CONST(member_type, struct_p, struct_offset)   \
+    (*(member_type const *) ((gconstpointer) ((const guint8*) (struct_p) + (glong) (struct_offset))))
+#endif
+
 const gchar *default_player_style = "square";
 
 typedef enum {
@@ -62,6 +70,9 @@ static Param game_params[] = {
 	{PARAM(domestic-trade, FIRST_VERSION, PARAM_BOOL, domestic_trade)},
 	{PARAM(num-players, FIRST_VERSION, PARAM_INT, num_players)},
 	{PARAM(sevens-rule, FIRST_VERSION, PARAM_INT, sevens_rule)},
+	{PARAM(use-dice-deck, V15, PARAM_BOOL, use_dice_deck)},
+	{PARAM(num-dice-decks, V15, PARAM_INT, num_dice_decks)},
+	{PARAM(num-removed-dice-cards, V15, PARAM_INT, num_removed_dice_cards)},
 	{PARAM(victory-points, FIRST_VERSION, PARAM_INT, victory_points)},
 	{PARAM(check-victory-at-end-of-turn, FIRST_VERSION, PARAM_BOOL, check_victory_at_end_of_turn)},
 	{PARAM(num-roads, FIRST_VERSION, PARAM_INT, num_build_type[BUILD_ROAD])},
@@ -92,17 +103,21 @@ GameParams *params_new(void)
 	GameParams *params;
 
 	params = g_malloc0(sizeof(*params));
+	params->num_dice_decks = 2;
+	params->num_removed_dice_cards = 5;
+
 	return params;
 }
 
 void params_free(GameParams * params)
 {
-	if (params == NULL)
-		return;
-	gint idx;
+	guint idx;
 	gchar *str;
 	GArray *int_list;
 
+	if (params == NULL)
+		return;
+
 	for (idx = 0; idx < G_N_ELEMENTS(game_params); idx++) {
 		Param *param = game_params + idx;
 
@@ -132,16 +147,16 @@ void params_free(GameParams * params)
 	g_free(params);
 }
 
-static gchar *skip_space(gchar * str)
+static const gchar *skip_space(const gchar * str)
 {
 	while (isspace(*str))
 		str++;
 	return str;
 }
 
-static gboolean match_word(gchar ** str, const gchar * word)
+static gboolean match_word(const gchar ** str, const gchar * word)
 {
-	gint word_len;
+	size_t word_len;
 
 	word_len = strlen(word);
 	if (strncmp(*str, word, word_len) == 0) {
@@ -190,7 +205,7 @@ gchar *format_int_list(const gchar * name, GArray * array)
 {
 	gchar *old;
 	gchar *str;
-	int idx;
+	guint idx;
 
 	if (array == NULL)
 		return NULL;
@@ -226,7 +241,7 @@ struct nosetup_t {
 
 static gboolean find_no_setup(const Hex * hex, gpointer closure)
 {
-	gint idx;
+	guint idx;
 	struct nosetup_t *data = closure;
 	for (idx = 0; idx < G_N_ELEMENTS(hex->nodes); ++idx) {
 		const Node *node = hex->nodes[idx];
@@ -242,14 +257,14 @@ static gboolean find_no_setup(const Hex * hex, gpointer closure)
 	return FALSE;
 }
 
-void params_write_lines(GameParams * params, ClientVersionType version,
-			gboolean write_secrets, WriteLineFunc func,
-			gpointer user_data)
+void params_write_lines(const GameParams * params,
+			ClientVersionType version, gboolean write_secrets,
+			WriteLineFunc func, gpointer user_data)
 {
-	gint idx;
+	guint idx;
 	gint y;
 	gchar *buff;
-	gchar *str;
+	const gchar *str;
 
 	for (idx = 0; idx < G_N_ELEMENTS(game_params); idx++) {
 		Param *param = game_params + idx;
@@ -262,21 +277,29 @@ void params_write_lines(GameParams * params, ClientVersionType version,
 			case PARAM_SINGLE_LINE:
 			case PARAM_MULTIPLE_LINES:
 				str =
-				    G_STRUCT_MEMBER(gchar *, params,
-						    param->offset);
+				    G_STRUCT_MEMBER_CONST(gchar *, params,
+							  param->offset);
 				if (!str || strlen(str) < 1) {
 					continue;
 				};
 				break;
 			case PARAM_INT:
-				if (G_STRUCT_MEMBER
+				if (G_STRUCT_MEMBER_CONST
 				    (gint, params, param->offset) < 1) {
 
 					continue;
 				};
+				/* Sub-parameters to use-dice-deck need never
+				 * to be sent to older clients, the message
+				 * about use-dice-deck will suffice */
+				if (!strcmp(param->name, "num-dice-decks")
+				    || !strcmp(param->name,
+					       "num-removed-dice-cards")) {
+					continue;
+				}
 				break;
 			case PARAM_BOOL:
-				if (!G_STRUCT_MEMBER
+				if (!G_STRUCT_MEMBER_CONST
 				    (gboolean, params, param->offset)) {
 					continue;
 				}
@@ -284,10 +307,9 @@ void params_write_lines(GameParams * params, ClientVersionType version,
 			case PARAM_INTLIST:
 				buff =
 				    format_int_list(param->name,
-						    G_STRUCT_MEMBER(GArray
-								    *,
-								    params,
-								    param->offset));
+						    G_STRUCT_MEMBER_CONST
+						    (GArray *, params,
+						     param->offset));
 				if (buff == NULL) {
 					continue;
 				};
@@ -306,8 +328,8 @@ void params_write_lines(GameParams * params, ClientVersionType version,
 		switch (param->type) {
 		case PARAM_SINGLE_LINE:
 			str =
-			    G_STRUCT_MEMBER(gchar *, params,
-					    param->offset);
+			    G_STRUCT_MEMBER_CONST(gchar *, params,
+						  param->offset);
 			if (!str)
 				continue;
 			buff = g_strdup_printf("%s %s", param->name, str);
@@ -316,8 +338,8 @@ void params_write_lines(GameParams * params, ClientVersionType version,
 			break;
 		case PARAM_MULTIPLE_LINES:
 			str =
-			    G_STRUCT_MEMBER(gchar *, params,
-					    param->offset);
+			    G_STRUCT_MEMBER_CONST(gchar *, params,
+						  param->offset);
 			if (str) {
 				gchar **strv;
 				gchar **strv_it;
@@ -337,15 +359,15 @@ void params_write_lines(GameParams * params, ClientVersionType version,
 			break;
 		case PARAM_INT:
 			buff = g_strdup_printf("%s %d", param->name,
-					       G_STRUCT_MEMBER(gint,
-							       params,
-							       param->
-							       offset));
+					       G_STRUCT_MEMBER_CONST(gint,
+								     params,
+								     param->
+								     offset));
 			func(user_data, buff);
 			g_free(buff);
 			break;
 		case PARAM_BOOL:
-			if (G_STRUCT_MEMBER
+			if (G_STRUCT_MEMBER_CONST
 			    (gboolean, params, param->offset)) {
 				func(user_data, param->name);
 			}
@@ -353,10 +375,10 @@ void params_write_lines(GameParams * params, ClientVersionType version,
 		case PARAM_INTLIST:
 			buff =
 			    format_int_list(param->name,
-					    G_STRUCT_MEMBER(GArray *,
-							    params,
-							    param->
-							    offset));
+					    G_STRUCT_MEMBER_CONST(GArray *,
+								  params,
+								  param->
+								  offset));
 			/* Don't send empty intlists */
 			if (buff != NULL) {
 				func(user_data, buff);
@@ -386,9 +408,9 @@ void params_write_lines(GameParams * params, ClientVersionType version,
 	}
 }
 
-gboolean params_load_line(GameParams * params, gchar * line)
+gboolean params_load_line(GameParams * params, const gchar * line)
 {
-	gint idx;
+	guint idx;
 
 	if (params->map == NULL)
 		params->map = map_new();
@@ -462,9 +484,11 @@ gboolean params_load_line(GameParams * params, gchar * line)
 					    param->offset);
 			if (str) {
 				gchar *copy;
+				gchar *line2 = g_strdup(line);
 				copy =
 				    g_strconcat(str, "\n",
-						g_strchomp(line), NULL);
+						g_strchomp(line2), NULL);
+				g_free(line2);
 				g_free(str);
 				str = copy;
 			} else {
@@ -517,7 +541,7 @@ gboolean params_load_line(GameParams * params, gchar * line)
 gboolean read_line_from_file(gchar ** line, FILE * f)
 {
 	gchar part[512];
-	gint len;
+	size_t len;
 
 	if (fgets(part, sizeof(part), f) == NULL)
 		return FALSE;
@@ -570,19 +594,13 @@ GameParams *params_load_file(const gchar * fname)
 
 GameParams *params_copy(const GameParams * params)
 {
-	/* Copy the const parameter to a non-const version, because
-	 * G_STRUCT_MEMBER doesn't want const values.  Note that this
-	 * variable does not own its pointers, that is, they don't have to
-	 * be freed when it goes out of scope. */
-	GameParams nonconst;
 	GameParams *copy;
-	gint idx;
+	guint idx;
 	gchar *buff;
 
 	if (params == NULL)
 		return NULL;
 
-	memcpy(&nonconst, params, sizeof(GameParams));
 	copy = params_new();
 	copy->map = map_copy(params->map);
 
@@ -594,26 +612,26 @@ GameParams *params_copy(const GameParams * params)
 		case PARAM_MULTIPLE_LINES:
 			G_STRUCT_MEMBER(gchar *, copy, param->offset)
 			    =
-			    g_strdup(G_STRUCT_MEMBER
-				     (gchar *, &nonconst, param->offset));
+			    g_strdup(G_STRUCT_MEMBER_CONST
+				     (gchar *, params, param->offset));
 			break;
 		case PARAM_INT:
 			G_STRUCT_MEMBER(gint, copy, param->offset)
-			    = G_STRUCT_MEMBER(gint, &nonconst,
-					      param->offset);
+			    = G_STRUCT_MEMBER_CONST(gint, params,
+						    param->offset);
 			break;
 		case PARAM_BOOL:
 			G_STRUCT_MEMBER(gboolean, copy, param->offset)
-			    = G_STRUCT_MEMBER(gboolean, &nonconst,
-					      param->offset);
+			    = G_STRUCT_MEMBER_CONST(gboolean, params,
+						    param->offset);
 			break;
 		case PARAM_INTLIST:
 			buff =
 			    format_int_list("",
-					    G_STRUCT_MEMBER(GArray *,
-							    &nonconst,
-							    param->
-							    offset));
+					    G_STRUCT_MEMBER_CONST(GArray *,
+								  params,
+								  param->
+								  offset));
 			if (buff != NULL) {
 				G_STRUCT_MEMBER(GArray *, copy,
 						param->offset) =
@@ -647,18 +665,20 @@ static void append_to_string(gpointer base, const gchar * additional_text)
 gboolean params_is_equal(const GameParams * params1,
 			 const GameParams * params2)
 {
-	gint idx;
+	gint i;
+	guint idx;
 	gchar *buff1;
 	gchar *buff2;
 	gboolean is_different;
+	struct nosetup_t tmp;
 
 	/* Compare the map */
 	if (params1->map->y_size != params2->map->y_size) {
 		return FALSE;
 	};
-	for (idx = 0; idx < params1->map->y_size; idx++) {
-		buff1 = map_format_line(params1->map, TRUE, idx);
-		buff2 = map_format_line(params2->map, TRUE, idx);
+	for (i = 0; i < params1->map->y_size; i++) {
+		buff1 = map_format_line(params1->map, TRUE, i);
+		buff2 = map_format_line(params2->map, TRUE, i);
 
 		is_different = g_strcmp0(buff1, buff2) != 0;
 		g_free(buff1);
@@ -677,7 +697,6 @@ gboolean params_is_equal(const GameParams * params1,
 		return FALSE;
 	}
 
-	struct nosetup_t tmp;
 	buff1 = NULL;
 	tmp.user_data = &buff1;
 	tmp.func = append_to_string;
@@ -694,16 +713,6 @@ gboolean params_is_equal(const GameParams * params1,
 	}
 
 	/* Compare the game parameters */
-
-	/* Copy the const parameter to a non-const version, because
-	 * G_STRUCT_MEMBER doesn't want const values.  Note that this
-	 * variable does not own its pointers, that is, they don't have to
-	 * be freed when it goes out of scope. */
-	GameParams nonconst1;
-	GameParams nonconst2;
-	memcpy(&nonconst1, params1, sizeof(GameParams));
-	memcpy(&nonconst2, params2, sizeof(GameParams));
-
 	for (idx = 0; idx < G_N_ELEMENTS(game_params); idx++) {
 		const Param *param = game_params + idx;
 
@@ -711,44 +720,42 @@ gboolean params_is_equal(const GameParams * params1,
 		case PARAM_SINGLE_LINE:
 		case PARAM_MULTIPLE_LINES:
 			if (g_strcmp0
-			    (G_STRUCT_MEMBER
-			     (gchar *, &nonconst1, param->offset),
-			     G_STRUCT_MEMBER(gchar *, &nonconst2,
-					     param->offset)) != 0) {
+			    (G_STRUCT_MEMBER_CONST
+			     (gchar *, params1, param->offset),
+			     G_STRUCT_MEMBER_CONST(gchar *, params2,
+						   param->offset)) != 0) {
 				return FALSE;
 			}
 			break;
 		case PARAM_INT:
-			if (G_STRUCT_MEMBER
-			    (gint, &nonconst1,
-			     param->offset) != G_STRUCT_MEMBER(gint,
-							       &nonconst2,
-							       param->offset))
+			if (G_STRUCT_MEMBER_CONST
+			    (gint, params1,
+			     param->offset) != G_STRUCT_MEMBER_CONST(gint,
+								     params2,
+								     param->offset))
 			{
 				return FALSE;
 			}
 			break;
 		case PARAM_BOOL:
-			if (G_STRUCT_MEMBER
-			    (gboolean, &nonconst1,
-			     param->offset) != G_STRUCT_MEMBER(gboolean,
-							       &nonconst2,
-							       param->offset))
-			{
+			if (G_STRUCT_MEMBER_CONST
+			    (gboolean, params1,
+			     param->offset) !=
+			    G_STRUCT_MEMBER_CONST(gboolean, params2,
+						  param->offset)) {
 				return FALSE;
 			}
 			break;
 		case PARAM_INTLIST:
 			buff1 = format_int_list("",
-						G_STRUCT_MEMBER(GArray *,
-								&nonconst1,
-								param->
-								offset));
+						G_STRUCT_MEMBER_CONST
+						(GArray *, params1,
+						 param->offset));
 			buff2 =
 			    format_int_list("",
-					    G_STRUCT_MEMBER(GArray *,
-							    &nonconst2,
-							    param->offset));
+					    G_STRUCT_MEMBER_CONST(GArray *,
+								  params2,
+								  param->offset));
 
 			is_different = g_strcmp0(buff1, buff2) != 0;
 			g_free(buff1);
@@ -800,7 +807,7 @@ static void write_one_line(gpointer user_data, const gchar * line)
 	fprintf(fp, "%s\n", line);
 }
 
-gboolean params_write_file(GameParams * params, const gchar * fname)
+gboolean params_write_file(const GameParams * params, const gchar * fname)
 {
 	FILE *fp;
 
@@ -816,19 +823,42 @@ gboolean params_write_file(GameParams * params, const gchar * fname)
 	return TRUE;
 }
 
+/* Conversions for ClientVersionType. Keep the newest version on top. */
+static struct ClientVersionTypeConversion {
+	ClientVersionType type;
+	const gchar *string;
+} client_version_type_conversions[] = {
+	{
+	V15, "15"}, {
+	V14, "14"}, {
+	V0_12, "0.12"}, {
+	V0_11, "0.11"}, {
+	V0_10, "0.10"}
+};
+
 ClientVersionType client_version_type_from_string(const gchar * cvt)
 {
-	if (!strcmp(cvt, "0.10")) {
-		return V0_10;
-	} else if (!strcmp(cvt, "0.11")) {
-		return V0_11;
-	} else if (!strcmp(cvt, "0.12")) {
-		return V0_12;
-	} else if (!strcmp(cvt, "14")) {
-		return V14;
-	} else {
-		return UNKNOWN_VERSION;
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS(client_version_type_conversions); i++) {
+		if (!strcmp
+		    (cvt, client_version_type_conversions[i].string)) {
+			return client_version_type_conversions[i].type;
+		}
 	}
+	return UNKNOWN_VERSION;
+}
+
+const gchar *client_version_type_to_string(ClientVersionType cvt)
+{
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS(client_version_type_conversions); i++) {
+		if (cvt == client_version_type_conversions[i].type) {
+			return client_version_type_conversions[i].string;
+		}
+	}
+	return "unknown";
 }
 
 gboolean can_client_connect_to_server(ClientVersionType client_version,
@@ -855,11 +885,15 @@ WinnableState params_check_winnable_state(const GameParams * params,
 					  gchar ** win_message,
 					  gchar ** point_specification)
 {
-	gint target, building, development;
-	gint road, army;
-	gint idx;
+	guint target;
+	guint building;
+	guint development;
+	guint road;
+	guint army;
+	guint idx;
 	WinnableState return_value;
-	gint total_island, max_island;
+	gint total_island;
+	guint max_island;
 	guint number_of_islands;
 
 	if (params == NULL) {
@@ -899,7 +933,7 @@ WinnableState params_check_winnable_state(const GameParams * params,
 	    && params->island_discovery_bonus->len > 0
 	    && (params->num_build_type[BUILD_SHIP] +
 		params->num_build_type[BUILD_BRIDGE] > 0)) {
-		gint i;
+		guint i;
 		for (i = 0; i < number_of_islands - 1; i++) {
 			total_island +=
 			    g_array_index(params->island_discovery_bonus,
@@ -909,7 +943,7 @@ WinnableState params_check_winnable_state(const GameParams * params,
 					   island_discovery_bonus->len - 1,
 					   i));
 			/* The island score can be negative */
-			if (max_island < total_island)
+			if ((gint) max_island < total_island)
 				max_island = total_island;
 		}
 	}
@@ -937,16 +971,21 @@ WinnableState params_check_winnable_state(const GameParams * params,
 	    g_strdup_printf(_(""
 			      "Required victory points: %d\n"
 			      "Points obtained by building all: %d\n"
-			      "Points in development cards: %d\n"
+			      "Points in development cards: %u\n"
 			      "Longest road/largest army: %d+%d\n"
 			      "Maximum island discovery bonus: %d\n"
 			      "Total: %d"), target, building, development,
 			    road, army, max_island,
-			    building + development + road + army +
+			    building + (gint) development + road + army +
 			    max_island);
 	return return_value;
 }
 
+gboolean params_game_is_unstartable(const GameParams * params)
+{
+	return params->num_build_type[BUILD_SETTLEMENT] < 2;
+}
+
 PlayerType determine_player_type(const gchar * style)
 {
 	gchar **style_parts;
@@ -991,9 +1030,9 @@ static const gchar *resource_types[] = {
 	"lumber"
 };
 
-static gint get_num(const gchar * str, gint * num)
+static ssize_t get_num(const gchar * str, gint * num)
 {
-	gint len = 0;
+	ssize_t len = 0;
 	gboolean is_negative = FALSE;
 
 	if (*str == '-') {
@@ -1011,10 +1050,22 @@ static gint get_num(const gchar * str, gint * num)
 	return len;
 }
 
-gint game_scanf(const gchar * line, const gchar * fmt, ...)
+static ssize_t get_unum(const gchar * str, guint * num)
+{
+	ssize_t len = 0;
+
+	*num = 0;
+	while (isdigit(*str)) {
+		*num = *num * 10 + (guint) (*str++ - '0');
+		len++;
+	}
+	return len;
+}
+
+ssize_t game_scanf(const gchar * line, const gchar * fmt, ...)
 {
 	va_list ap;
-	gint offset;
+	ssize_t offset;
 
 	va_start(ap, fmt);
 	offset = game_vscanf(line, fmt, ap);
@@ -1023,15 +1074,16 @@ gint game_scanf(const gchar * line, const gchar * fmt, ...)
 	return offset;
 }
 
-gint game_vscanf(const gchar * line, const gchar * fmt, va_list ap)
+ssize_t game_vscanf(const gchar * line, const gchar * fmt, va_list ap)
 {
-	gint offset = 0;
+	ssize_t offset = 0;
 
 	while (*fmt != '\0' && line[offset] != '\0') {
 		gchar **str;
 		gint *num;
+		guint *unum;
 		gint idx;
-		gint len;
+		ssize_t len;
 		BuildType *build_type;
 		Resource *resource;
 
@@ -1048,7 +1100,7 @@ gint game_vscanf(const gchar * line, const gchar * fmt, va_list ap)
 		case 'S':	/* string from current position to end of line */
 			str = va_arg(ap, gchar **);
 			*str = g_strdup(line + offset);
-			offset += strlen(*str);
+			offset += (ssize_t) strlen(*str);
 			break;
 		case 'd':	/* integer */
 			num = va_arg(ap, gint *);
@@ -1057,6 +1109,13 @@ gint game_vscanf(const gchar * line, const gchar * fmt, va_list ap)
 				return -1;
 			offset += len;
 			break;
+		case 'u':	/* unsigned integer */
+			unum = va_arg(ap, guint *);
+			len = get_unum(line + offset, unum);
+			if (len == 0)
+				return -1;
+			offset += len;
+			break;
 		case 'B':	/* build type */
 			build_type = va_arg(ap, BuildType *);
 			if (strncmp(line + offset, "road", 4) == 0) {
@@ -1106,8 +1165,10 @@ gint game_vscanf(const gchar * line, const gchar * fmt, va_list ap)
 			resource = va_arg(ap, Resource *);
 			for (idx = 0; idx < NO_RESOURCE; idx++) {
 				const gchar *type = resource_types[idx];
-				len = strlen(type);
-				if (strncmp(line + offset, type, len) == 0) {
+				len = (ssize_t) strlen(type);
+				if (strncmp
+				    (line + offset, type,
+				     (size_t) len) == 0) {
 					offset += len;
 					*resource = idx;
 					break;
@@ -1149,14 +1210,16 @@ gchar *game_vprintf(const gchar * fmt, va_list ap)
 
 	while (*fmt != '\0') {
 		gchar *pos = strchr(fmt, '%');
+		gchar *text_without_format;
+
 		if (pos == NULL) {
 			buff_append(result, "%s", fmt);
 			break;
 		}
 		/* add format until next % to result */
-		result = g_realloc(result, strlen(result) + pos - fmt + 1);
-		result[strlen(result) + pos - fmt] = '\0';
-		memcpy(&result[strlen(result)], fmt, pos - fmt);
+		text_without_format = g_strndup(fmt, (gsize) (pos - fmt));
+		buff_append(result, "%s", text_without_format);
+		g_free(text_without_format);
 		fmt = pos + 1;
 
 		switch (*fmt++) {
@@ -1170,6 +1233,9 @@ gchar *game_vprintf(const gchar * fmt, va_list ap)
 		case 'D':	/* development card type */
 			buff_append(result, "%d", va_arg(ap, gint));
 			break;
+		case 'u':	/* unsigned integer */
+			buff_append(result, "%u", va_arg(ap, guint));
+			break;
 		case 'B':	/* build type */
 			build_type = va_arg(ap, BuildType);
 			switch (build_type) {
