@@ -3,7 +3,7 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003, 2006 Bas Wijnen <shevek@fmf.nl>
- * Copyright (C) 2007 Roland Clobus <rclobus@bigfoot.com>
+ * Copyright (C) 2007, 2013 Roland Clobus <rclobus@rclobus.nl>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/* Pioneers Console Server Adminstrator interface
+/* Pioneers Console Server Administrator interface
  *
  * The strings in the admin interface are intentionally not translated.
  * They would otherwise reflect the language of the server that is
@@ -33,11 +33,16 @@
 #include "admin.h"
 #include "game.h"
 #include "server.h"
+#include "network.h"
+#include "version.h"
 
 /* network administration functions */
-comm_info *_accept_info = NULL;
-
-gint admin_dice_roll = 0;
+static Game **admin_game;
+static gint admin_dice_roll = 0;
+static gchar *server_port = NULL;
+static gboolean register_server = TRUE;
+static GameParams *params = NULL;
+static Service *service = NULL;
 
 typedef enum {
 	BADCOMMAND,
@@ -47,6 +52,9 @@ typedef enum {
 	REGISTERSERVER,
 	NUMPLAYERS,
 	SEVENSRULE,
+	DICEDECK,
+	NUMDICEDECKS,
+	NUMREMOVEDDICECARDS,
 	VICTORYPOINTS,
 	RANDOMTERRAIN,
 	SETGAME,
@@ -54,49 +62,63 @@ typedef enum {
 	MESSAGE,
 	HELP,
 	INFO,
-	FIXDICE
+	FIXDICE,
+	GETBANK,
+	SETBANK,
+	GETASSETS,
+	SETASSETS
 } AdminCommandType;
 
+typedef enum {
+	NONEED,
+	NEEDPARAMS,
+	NEEDGAME
+} AdminRequirementType;
+
 typedef struct {
 	AdminCommandType type;
 	const gchar *command;
 	gboolean need_argument;
 	gboolean stop_server;
-	gboolean need_gameparam;
+	AdminRequirementType requirement;
 } AdminCommand;
 
 /* *INDENT-OFF* */
 static AdminCommand admin_commands[] = {
-	{ BADCOMMAND,     "",                    FALSE, FALSE, FALSE },
-	{ SETPORT,        "set-port",            TRUE,  TRUE,  TRUE  },
-	{ STARTSERVER,    "start-server",        FALSE, TRUE,  TRUE  },
-	{ STOPSERVER,     "stop-server",         FALSE, TRUE,  FALSE },
-	{ REGISTERSERVER, "set-register-server", TRUE,  TRUE,  FALSE },
-	{ NUMPLAYERS,     "set-num-players",     TRUE,  TRUE,  TRUE  },
-	{ SEVENSRULE,     "set-sevens-rule",     TRUE,  TRUE,  TRUE  },
-	{ VICTORYPOINTS,  "set-victory-points",  TRUE,  TRUE,  TRUE  },
-	{ RANDOMTERRAIN,  "set-random-terrain",  TRUE,  TRUE,  TRUE  },
-	{ SETGAME,        "set-game",            TRUE,  TRUE,  FALSE },
-	{ QUIT,           "quit",                FALSE, FALSE, FALSE },
-	{ MESSAGE,        "send-message",        TRUE,  FALSE, TRUE  },
-	{ HELP,           "help",                FALSE, FALSE, FALSE },
-	{ INFO,           "info",                FALSE, FALSE, FALSE },
-	{ FIXDICE,        "fix-dice",            TRUE,  FALSE, FALSE },
+	{ BADCOMMAND,          "",                    FALSE, FALSE, NONEED     },
+	{ SETPORT,             "set-port",            TRUE,  TRUE,  NEEDPARAMS },
+	{ STARTSERVER,         "start-server",        FALSE, TRUE,  NEEDPARAMS },
+	{ STOPSERVER,          "stop-server",         FALSE, TRUE,  NONEED },
+	{ REGISTERSERVER,      "set-register-server", TRUE,  TRUE,  NONEED },
+	{ NUMPLAYERS,          "set-num-players",     TRUE,  TRUE,  NEEDPARAMS },
+	{ SEVENSRULE,          "set-sevens-rule",     TRUE,  TRUE,  NEEDPARAMS },
+	{ DICEDECK,            "set-dice-deck",       TRUE,  TRUE,  NEEDPARAMS },
+	{ NUMDICEDECKS,        "set-num-dice-decks",  TRUE,  TRUE,  NEEDPARAMS },
+	{ NUMREMOVEDDICECARDS, "set-num-removed-dice-cards",
+	                                              TRUE,  TRUE,  NEEDPARAMS },
+
+	{ VICTORYPOINTS,       "set-victory-points",  TRUE,  TRUE,  NEEDPARAMS },
+	{ RANDOMTERRAIN,       "set-random-terrain",  TRUE,  TRUE,  NEEDPARAMS },
+	{ SETGAME,             "set-game",            TRUE,  TRUE,  NONEED     },
+	{ QUIT,                "quit",                FALSE, FALSE, NONEED     },
+	{ MESSAGE,             "send-message",        TRUE,  FALSE, NEEDGAME   },
+	{ HELP,                "help",                FALSE, FALSE, NONEED     },
+	{ INFO,                "info",                FALSE, FALSE, NONEED     },
+	{ FIXDICE,             "fix-dice",            TRUE,  FALSE, NEEDGAME   },
+	{ GETBANK,             "get-bank",            FALSE, FALSE, NEEDGAME   },
+	{ SETBANK,             "set-bank",            TRUE,  FALSE, NEEDGAME   },
+	{ GETASSETS,           "get-assets",          TRUE,  FALSE, NEEDGAME   },
+	{ SETASSETS,           "set-assets",          TRUE,  FALSE, NEEDGAME   },
 };
 /* *INDENT-ON* */
 
 /* parse 'line' and run the command requested */
-void admin_run_command(Session * admin_session, const gchar * line)
+static void admin_run_command(Session * admin_session, const gchar * line)
 {
 	const gchar *command_start;
 	gchar *command;
 	gchar *argument;
-	gint command_number;
-
-	static gchar *server_port = NULL;
-	static gboolean register_server = TRUE;
-	static GameParams *params = NULL;
-	static Game *game = NULL;
+	guint command_number;
 
 	if (!g_str_has_prefix(line, "admin")) {
 		net_printf(admin_session,
@@ -117,7 +139,7 @@ void admin_run_command(Session * admin_session, const gchar * line)
 	command_start = line;
 	while (*line && !g_ascii_isspace(*line))
 		++line;
-	command = g_strndup(command_start, line - command_start);
+	command = g_strndup(command_start, (gsize) (line - command_start));
 
 	if (*line) {
 		while (*line && g_ascii_isspace(*line))
@@ -144,17 +166,23 @@ void admin_run_command(Session * admin_session, const gchar * line)
 		net_printf(admin_session,
 			   "ERROR command '%s' needs an argument\n",
 			   command);
-	} else if (admin_commands[command_number].need_gameparam
+	} else if (admin_commands[command_number].requirement == NEEDPARAMS
 		   && NULL == params) {
+		net_printf(admin_session,
+			   "ERROR command '%s' needs valid game parameters\n",
+			   command);
+	} else if (admin_commands[command_number].requirement == NEEDGAME
+		   && (NULL == *admin_game
+		       || !server_is_running(*admin_game))) {
 		net_printf(admin_session,
 			   "ERROR command '%s' needs a valid game\n",
 			   command);
 	} else {
 		if (admin_commands[command_number].stop_server
-		    && server_is_running(game)) {
-			server_stop(game);
-			game_free(game);
-			game = NULL;
+		    && server_is_running(*admin_game)) {
+			server_stop(*admin_game);
+			game_free(*admin_game);
+			*admin_game = NULL;
 			net_write(admin_session, "INFO server stopped\n");
 		}
 		switch (admin_commands[command_number].type) {
@@ -170,24 +198,24 @@ void admin_run_command(Session * admin_session, const gchar * line)
 			break;
 		case STARTSERVER:
 			{
-				gchar *meta_server_name =
-				    get_meta_server_name(TRUE);
+				gchar *metaserver_name =
+				    get_metaserver_name(TRUE);
 				if (!server_port)
 					server_port =
 					    g_strdup
 					    (PIONEERS_DEFAULT_GAME_PORT);
-				if (game != NULL)
-					game_free(game);
-				game =
+				if (*admin_game != NULL)
+					game_free(*admin_game);
+				*admin_game =
 				    server_start(params, get_server_name(),
 						 server_port,
 						 register_server,
-						 meta_server_name, TRUE);
-				g_free(meta_server_name);
+						 metaserver_name, TRUE);
+				g_free(metaserver_name);
 			}
 			break;
 		case STOPSERVER:
-			server_stop(game);
+			server_stop(*admin_game);
 			break;
 		case REGISTERSERVER:
 			register_server = atoi(argument);
@@ -198,6 +226,16 @@ void admin_run_command(Session * admin_session, const gchar * line)
 		case SEVENSRULE:
 			cfg_set_sevens_rule(params, atoi(argument));
 			break;
+		case DICEDECK:
+			cfg_set_use_dice_deck(params, atoi(argument));
+			break;
+		case NUMDICEDECKS:
+			cfg_set_num_dice_decks(params, atoi(argument));
+			break;
+		case NUMREMOVEDDICECARDS:
+			cfg_set_num_removed_dice_cards(params,
+						       atoi(argument));
+			break;
 		case VICTORYPOINTS:
 			cfg_set_victory_points(params, atoi(argument));
 			break;
@@ -217,13 +255,13 @@ void admin_run_command(Session * admin_session, const gchar * line)
 		case QUIT:
 			net_close(admin_session);
 			/* Quit the server if the admin leaves */
-			if (!server_is_running(game))
+			if (!server_is_running(*admin_game))
 				exit(0);
 			break;
 		case MESSAGE:
 			g_strdelimit(argument, "|", '_');
-			if (server_is_running(game))
-				admin_broadcast(game, argument);
+			if (server_is_running(*admin_game))
+				admin_broadcast(*admin_game, argument);
 			break;
 		case HELP:
 			for (command_number = 1;
@@ -254,7 +292,7 @@ void admin_run_command(Session * admin_session, const gchar * line)
 				   register_server);
 			net_printf(admin_session,
 				   "INFO server running %d\n",
-				   server_is_running(game));
+				   server_is_running(*admin_game));
 			if (params) {
 				net_printf(admin_session, "INFO game %s\n",
 					   params->title);
@@ -270,6 +308,42 @@ void admin_run_command(Session * admin_session, const gchar * line)
 				net_printf(admin_session,
 					   "INFO sevens-rule %d\n",
 					   params->sevens_rule);
+				if (server_is_running(*admin_game)) {
+					gchar *s =
+					    game_printf("INFO bank %R\n",
+							(*admin_game)->bank_deck);
+					net_printf(admin_session, "%s", s);
+					g_free(s);
+
+					playerlist_inc_use_count
+					    (*admin_game);
+					GList *player =
+					    player_first_real(*admin_game);
+					while (player) {
+						Player *p = (Player *)
+						    player->data;
+						if (player_is_spectator
+						    (*admin_game,
+						     p->num)) {
+							s = game_printf
+							    ("INFO spectator %d\n",
+							     p->num);
+						} else {
+							s = game_printf
+							    ("INFO player %d assets %R\n",
+							     p->num,
+							     p->assets);
+						}
+						net_printf(admin_session,
+							   "%s", s);
+						g_free(s);
+						player =
+						    player_next_real
+						    (player);
+					}
+					playerlist_dec_use_count
+					    (*admin_game);
+				}
 			} else {
 				net_printf(admin_session,
 					   "INFO no game set\n");
@@ -283,13 +357,87 @@ void admin_run_command(Session * admin_session, const gchar * line)
 			admin_dice_roll = CLAMP(atoi(argument), 0, 12);
 			if (admin_dice_roll == 1)
 				admin_dice_roll = 0;
-			if (admin_dice_roll != 0)
+			if (admin_dice_roll != 0) {
+				(*admin_game)->is_manipulated = TRUE;
 				net_printf(admin_session,
 					   "INFO dice fixed to %d\n",
 					   admin_dice_roll);
-			else
+			} else
 				net_printf(admin_session,
 					   "INFO dice rolled normally\n");
+			break;
+		case SETBANK:
+			{
+				game_scanf(argument, "%R",
+					   &(*admin_game)->bank_deck);
+				(*admin_game)->is_manipulated = TRUE;
+			}
+			// FALL THROUGH
+		case GETBANK:
+			{
+				gchar *s = game_printf("INFO bank %R\n",
+						       (*admin_game)->bank_deck);
+				net_printf(admin_session, "%s", s);
+				g_free(s);
+			}
+			break;
+		case SETASSETS:
+			{
+				gint player_num;
+				gint assets[NO_RESOURCE];
+				Player *player;
+				gint i;
+
+				game_scanf(argument, "%d %R", &player_num,
+					   &assets);
+				player =
+				    player_by_num(*admin_game, player_num);
+				if (player != NULL
+				    && !player_is_spectator(*admin_game,
+							    player_num)) {
+					for (i = 0; i < NO_RESOURCE; i++) {
+						(*admin_game)->bank_deck[i]
+						    +=
+						    player->assets[i] -
+						    assets[i];
+						player->assets[i] =
+						    assets[i];
+					}
+				}
+				(*admin_game)->is_manipulated = TRUE;
+			}
+			// FALL THROUGH
+		case GETASSETS:
+			{
+				gint player_num;
+				Player *player;
+				game_scanf(argument, "%d", &player_num);
+				player =
+				    player_by_num(*admin_game, player_num);
+				if (player != NULL) {
+					if (player_is_spectator
+					    (*admin_game, player_num)) {
+						net_printf(admin_session,
+							   "INFO player %d is spectator\n",
+							   player_num);
+					} else {
+						gchar *s =
+						    game_printf
+						    ("INFO player %d assets %R\n",
+						     player_num,
+						     player->assets);
+						net_printf(admin_session,
+							   "%s", s);
+						g_free(s);
+					}
+				} else {
+					net_printf(admin_session,
+						   "INFO player %d not found\n",
+						   player_num);
+				}
+
+			}
+			break;
 		}
 	}
 	g_free(command);
@@ -297,14 +445,14 @@ void admin_run_command(Session * admin_session, const gchar * line)
 		g_free(argument);
 }
 
-/* network event handler, just like the one in meta.c, state.c, etc. */
-void admin_event(NetEvent event, Session * admin_session,
-		 const gchar * line)
+/* network event handler */
+static void admin_event(Session * ses, NetEvent event, const gchar * line,
+			G_GNUC_UNUSED gpointer user_data)
 {
 #ifdef PRINT_INFO
 	g_print
 	    ("admin_event: event = %#x, admin_session = %p, line = %s\n",
-	     event, admin_session, line);
+	     event, ses, line);
 #endif
 
 	switch (event) {
@@ -314,7 +462,7 @@ void admin_event(NetEvent event, Session * admin_session,
 #ifdef PRINT_INFO
 		g_print("admin_event: NET_READ: line = '%s'\n", line);
 #endif
-		admin_run_command(admin_session, line);
+		admin_run_command(ses, line);
 		break;
 	case NET_CLOSE:
 		/* connection has been closed */
@@ -322,11 +470,14 @@ void admin_event(NetEvent event, Session * admin_session,
 #ifdef PRINT_INFO
 		g_print("admin_event: NET_CLOSE\n");
 #endif
-		net_free(&admin_session);
+		net_free(&ses);
 		break;
 	case NET_CONNECT:
-		/* connect() succeeded -- shouldn't get here */
-
+		/* new connection was made */
+		net_printf(ses,
+			   "welcome to the pioneers admin connection, "
+			   "version %s\n", FULL_VERSION);
+		net_set_check_connection_alive(ses, 0);
 #ifdef PRINT_INFO
 		g_print("admin_event: NET_CONNECT\n");
 #endif
@@ -337,57 +488,34 @@ void admin_event(NetEvent event, Session * admin_session,
 #ifdef PRINT_INFO
 		g_print("admin_event: NET_CONNECT_FAIL\n");
 #endif
-		break;
-	default:
-		/* To kill a warning... */
+		net_free(&ses);
 		break;
 	}
 }
 
-/* accept a connection made to the admin port */
-void admin_connect(comm_info * admin_info)
-{
-	Session *admin_session;
-	gint new_fd;
-	gchar *location;
-
-	/* somebody connected to the administration port, so we... */
-
-	/* (1) create a new network session */
-	admin_session = net_new((NetNotifyFunc) admin_event, NULL);
-
-	/* (2) accept the connection into a new file descriptor */
-	new_fd = accept_connection(admin_info->fd, &location);
-
-	/* (3) tie the new file descriptor to the session we created earlier.
-	 * Don't use keepalive pings on this connection.  */
-	net_use_fd(admin_session, new_fd, FALSE);
-}
-
 /* set up the administration port */
-gboolean admin_listen(const gchar * port)
+gboolean admin_init(const gchar * port, Game ** game)
 {
 	gchar *error_message;
 
-	if (!_accept_info) {
-		_accept_info = g_malloc0(sizeof(comm_info));
+	admin_game = game;
+	if (*admin_game != NULL) {
+		params = params_copy((*admin_game)->params);
+		server_port = g_strdup((*admin_game)->server_port);
 	}
 
-	/* open up a socket on which to listen for connections */
-	_accept_info->fd = net_open_listening_socket(port, &error_message);
-	if (_accept_info->fd == -1) {
+	service =
+	    net_service_new(atoi(port), admin_event, NULL, &error_message);
+
+	if (!service) {
 		log_message(MSG_ERROR, "%s\n", error_message);
 		g_free(error_message);
 		return FALSE;
 	}
-#ifdef PRINT_INFO
-	g_print("admin_listen: fd = %d\n", _accept_info->fd);
-#endif
-
-	/* set up the callback to handle connections */
-	_accept_info->read_tag =
-	    driver->input_add_read(_accept_info->fd,
-				   (InputFunc) admin_connect,
-				   _accept_info);
 	return TRUE;
 }
+
+gint admin_get_dice_roll(void)
+{
+	return admin_dice_roll;
+}
